#ifndef YACE_OS_H
#define YACE_OS_H

#include "pre.h"
#include "config-all.h"
#include "global_macros.h"

class YACE_OS_Export YACE_Object_Manager_Base
{
public:
	YACE_Object_Manager_Base();
	virtual ~YACE_Object_Manager_Base();

	virtual int init() = 0;			// pure virtual	
	virtual int fini() = 0;
	
	enum Object_Manager_State
    {
		OBJ_MAN_UNINITIALIZED = 0,
			OBJ_MAN_INITIALIZING,
			OBJ_MAN_INITIALIZED,
			OBJ_MAN_SHUTTING_DOWN,
			OBJ_MAN_SHUT_DOWN
    };

protected:
	int starting_up_i();
	int shutting_down_();

	// state
	Object_Manager_State object_manager_state_;

private:
	YACE_Object_Manager_Base(const YACE_Object_Manager_Base&);
	YACE_Object_Manager_Base& operator=(const YACE_Object_Manager_Base&);
};

//# if defined (YACE_HAS_SIG_C_FUNC)
//extern "C" {
//# endif /* ACE_HAS_SIG_C_FUNC */
typedef void (*YACE_CLEANUP_FUNC)(void *object, void *param) /* throw () */;
//# if defined (ACE_HAS_SIG_C_FUNC)
//}
//# endif /* ACE_HAS_SIG_C_FUNC */

class YACE_OS_Export YACE_Cleanup
{
public:
	YACE_Cleanup();			// no-op
	virtual ~YACE_Cleanup();

	/// Cleanup method that, by default, simply deletes itself.
	virtual void cleanup (void *param = 0);
};

class YACE_OS_Export YACE_Cleanup_Info
{
public:
  /// Default constructor.
  YACE_Cleanup_Info (void);

  /// Equality operator.
  int operator== (const YACE_Cleanup_Info &o) const;

  /// Inequality operator.
  int operator!= (const YACE_Cleanup_Info &o) const;

  /// Point to object that gets passed into the <cleanup_hook_>.
  void *object_;

  /// Cleanup hook that gets called back.
  YACE_CLEANUP_FUNC cleanup_hook_;

  /// Parameter passed to the <cleanup_hook_>.
  void *param_;
};

class YACE_Cleanup_Info_Node;

class YACE_OS_Export YACE_OS_Exit_Info
{
public:
	YACE_OS_Exit_Info();
	virtual ~YACE_OS_Exit_Info();

	int at_exit_i (void *object, YACE_CLEANUP_FUNC cleanup_hook, void *param);
	int find (void *object);
	
	void call_hooks ();
	
private:
	YACE_Cleanup_Info_Node *registered_objects_;
	
};


#endif // YACE_OS_H